﻿using MemoryUtil;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net.Http;
using System.Runtime.CompilerServices;

namespace ConsoleLareFile
{
    class Program
    {
        //examples taken from: https://www.newtonsoft.com/json/help/html/Performance.htm
        static void Main(string[] args)
        {
            byte[] bytes = ReadFile();
            int iterations = 10;

            var timeSpan = Measure(() =>
            {
                for (int i = 0; i < iterations; i++)
                {
                    using (var memoryStream = new MemoryStream(bytes))
                    using (var streamReader = new StreamReader(memoryStream))
                    {
                        // read the json into a string
                        // string could potentially be very large and cause memory problems
                        string json = streamReader.ReadToEnd();
                        var example1 = JsonConvert.DeserializeObject<IList<RootObject>>(json);
                    }
                }
            });

            var timeSpan2 = Measure(() =>
            {
                for (int i = 0; i < iterations; i++)
                {
                    using (Stream s = new MemoryStream(bytes))
                    using (StreamReader sr = new StreamReader(s))
                    using (JsonReader reader = new JsonTextReader(sr))
                    {
                        // read the json from a stream
                        // json size doesn't matter because only a small piece is read at a time from the HTTP request
                        JsonSerializer serializer = new JsonSerializer();
                        var p = serializer.Deserialize<IList<RootObject>>(reader);
                    }
                }
            });

            var percent = (timeSpan - timeSpan2) / timeSpan * 100;
            var calc = timeSpan2 / timeSpan;
            Console.WriteLine($"x{calc} %{percent}");

            Console.ReadLine();
        }

        static GCCounter _gcCounter = new GCCounter();

        private static TimeSpan Measure(Action action)
        {
            var sw = new Stopwatch();
            _gcCounter.TakeSnapshot();
            sw.Start();

            action();

            sw.Stop();
            _gcCounter.TakeSnapshot();
            _gcCounter.PrintToConsole(sw.Elapsed.ToString(), false);
            var timeSpan = sw.Elapsed;

            return timeSpan;
        }

        private static byte[] ReadFile()
        {
            var path = Path.GetDirectoryName(new System.Uri(new Program().GetType().Assembly.GetName().CodeBase).LocalPath) + @"\large.json";

            byte[] bytes = File.ReadAllBytes(path);

            //string url = "http://www.test.com/large.json";
            //using (var client = new HttpClient())
            //{
            //    using (var response = client.GetAsync(url, HttpCompletionOption.ResponseHeadersRead).Result)
            //    {
            //        var stream = response.Content.ReadAsStreamAsync().Result;
            //        bytes = ToByteArray(stream);
            //    }
            //}

            return bytes;
        }

        public static byte[] ToByteArray(Stream stream)
        {
            var buffer = new byte[32768];

            using (var ms = new MemoryStream())
            {
                while (true)
                {
                    var read = stream.Read(buffer, 0, buffer.Length);

                    if (read <= 0)
                    {
                        return ms.ToArray();
                    }

                    ms.Write(buffer, 0, read);
                }
            }
        }
    }        
}
