﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;

namespace ConsoleTestRuntime
{
    public class FasterSpaceship : ISpaceship
    {
        private List<Traveler> _travelersSortedByFirstName;
        private List<Traveler> _travelersSortedByAge;
        private IComparer<Traveler> _compareByFirstName = new CompareByFirstName();
        private IComparer<Traveler> _compareByAge = new CompareByAge();
        public FasterSpaceship()
        {
            _travelersSortedByFirstName = new List<Traveler>();

            for (int i = 0; i < 5000; i++)
            {
                _travelersSortedByFirstName.Add(new Traveler()
                {
                    Id = i,
                    FirstName = "FirstName_" + i,
                    LastName = "LastNane_" + i,
                    Age = i
                });
            }

            _travelersSortedByFirstName.Sort((a,b)=> { return string.Compare(a.FirstName, b.FirstName, CultureInfo.InvariantCulture, CompareOptions.OrdinalIgnoreCase); });
            _travelersSortedByAge = _travelersSortedByFirstName.ToList();
            _travelersSortedByAge.Sort((a, b) => {
                if (a.Age > b.Age)
                    return 1;
                if (a.Age < b.Age)
                    return -1;
                return 0;
            });
        }
        
        /// <summary>
        ///     Tries to find any user that is adult.
        /// </summary>
        /// <remarks>
        ///     This method approaches an O(logn) operation.
        /// </remarks>
        /// <returns>
        ///     True if the item was found.
        /// </returns>
        public bool HasAdult()
        {
            var hasAdult = _travelersSortedByAge.BinarySearch(new Traveler() { Age = 4000 }, _compareByAge) > -1;
            return hasAdult;
        }

        /// <summary>
        ///     Returns list of adult users.
        /// </summary>
        /// <remarks>
        ///     This method approaches an O(N) operation.
        /// </remarks>
        public List<Traveler> GetAdults()
        {
            var adults = _travelersSortedByFirstName.Where(p => p.Age > 4000).ToList();
            return adults;
        }

        /// <summary>
        ///     Tries to find user with the same first name.
        /// </summary>
        /// <param name="name">The firstName.</param>
        /// <remarks>
        ///     This method approaches an O(logn) operation.
        /// </remarks>
        /// <returns>
        ///     True if the name was found.
        /// </returns>
        public bool HasTraveler(string name)
        {
            int index = _travelersSortedByFirstName.BinarySearch(new Traveler() { FirstName = name }, _compareByFirstName);
            if (index > -1)
            {
                return true;
            }

            return false;
        }
    }
}
