﻿using System;
using System.Diagnostics;

namespace ConsoleTestRuntime
{
    class Program
    {
        static void Main(string[] args)
        {
            WarmUp();

            int iterations = 100;
            RunTest(iterations);

            iterations = 1000;
            RunTest(iterations);

            iterations = 10000;
            RunTest(iterations);

            Console.ReadLine();
        }

        private static TimeSpan Measure(int iterations, ISpaceship spaceship)
        {
            var sw = Stopwatch.StartNew();

            for (int i = 1; i < iterations; i++)
            {
                if (spaceship.HasAdult() && !spaceship.HasTraveler("dsf")) { }
            }

            sw.Stop();

            return sw.Elapsed;
        }

        private static void RunTest(int iterations)
        {
            Console.WriteLine($"Run number of iterations: {iterations}");
            var timeSpanRegular = RunRegular(iterations);
            var timeSpanFaster = RunImproved(iterations);

            Console.WriteLine($"Regular:\t{timeSpanRegular}");
            Console.WriteLine($"Faster:\t\t{timeSpanFaster}\tx{timeSpanRegular / timeSpanFaster}");
        }

        private static TimeSpan RunRegular(int iterations)
        {
            var repository = new Spaceship();
            return Measure(iterations, repository);
        }

        private static TimeSpan RunImproved(int iterations)
        {
            var repository = new FasterSpaceship();
            return Measure(iterations, repository);
        }

        private static void WarmUp()
        {
            Console.WriteLine("Warm up");
            RunRegular(10);
            RunImproved(10);
        }
    }
}
