﻿using ConsoleTestRuntimeArchitectureChanges.Castle;
using ConsoleTestRuntimeArchitectureChanges.Improved;
using Microsoft.Extensions.Caching.Memory;
using Castle.Windsor.MsDependencyInjection;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Diagnostics;

namespace ConsoleTestRuntimeArchitectureChanges
{
    class Program
    {
        static void Main(string[] args)
        {
            var serviceProvider = RegisterServices();

            using (var scope = serviceProvider.CreateScope())
            {
                var spaceship = scope.ServiceProvider.GetService<ISpaceship>();

                spaceship.HasTraveler("tesrt");
            }

            Console.ReadLine();
        }

        private static IServiceProvider RegisterServices()
        {
            #region RegisterMemoryCacheLogger
            var services = new ServiceCollection();

            services.AddMemoryCache();
            services.AddSingleton(new LoggerFactory()
            .AddConsole()
            .AddDebug());

            services.AddLogging();
            #endregion

            services.AddScoped<ITravelerValidation, TravelerValidation>();

            // Solution 1
            services.AddScoped<ISpaceship, SpaceshipBeforeChanges>();
            var serviceProvider = services.BuildServiceProvider();

            // Solution 2
            //RegisterFactory(services);
            //var serviceProvider = services.BuildServiceProvider();

            //Solution 3
            //RegisterCastleFactory(services);
            //var serviceProvider = GetCastleResolver(services);

            return serviceProvider;
        }

        public static IServiceProvider GetCastleResolver(IServiceCollection services)
        {

            var resolver = new CastleServiceResolver(services);
            var container = resolver.GetServiceProvider();
            var kernel = resolver.GetWindsor().Kernel;
            kernel.Register(new RegisterInterceptors());


            return container;
        }

        private static void RegisterCastleFactory(IServiceCollection services)
        {
            var spaceshipFactory = new Func<IServiceProvider, ISpaceship>((provider) =>
            {
                var spaceship = new Spaceship();
                var travelerValidation = provider.GetService<ITravelerValidation>();

                var validation = new SpaceshipValidation(spaceship, travelerValidation);

                var logger = provider.GetService<ILogger<Spaceship>>();
                var logging = new SpaceshipLogging(validation, logger, travelerValidation);

                return logging;
            });

            services.AddScoped<ISpaceship>(spaceshipFactory);
        }

        private static void RegisterFactory(ServiceCollection services)
        {
            var spaceshipFactory = new Func<IServiceProvider, ISpaceship>((provider) =>
            {
                var spaceship = new Spaceship();
                var travelerValidation = provider.GetService<ITravelerValidation>();

                var cache = provider.GetService<IMemoryCache>();
                var caching = new SpaceshipCachig(spaceship, cache);

                var validation = new SpaceshipValidation(caching, travelerValidation);

                var logger = provider.GetService<ILogger<Spaceship>>();
                var logging = new SpaceshipLogging(validation, logger, travelerValidation);

                var profiling = new SpaceshipProfiling(logging);

                return profiling;
            });

            services.AddScoped<ISpaceship>(spaceshipFactory);
        }
    }
}
