﻿using ConsoleTestRuntimeArchitectureChanges.Improved;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;

namespace ConsoleTestRuntimeArchitectureChanges
{
    public class SpaceshipBeforeChanges : ISpaceship
    {
        private List<Traveler> _traveler;
        private IMemoryCache _cache;
        private ILogger<SpaceshipBeforeChanges> _logger;
        private ITravelerValidation _validation;
        public SpaceshipBeforeChanges(
            ILogger<SpaceshipBeforeChanges> logger, 
            IMemoryCache cache, 
            ITravelerValidation travelerValidation)
        {
            _logger = logger;
            _cache = cache;
            _validation = travelerValidation;
            #region init
            _traveler = new List<Traveler>();

            for (int i = 0; i < 5000; i++)
            {
                _traveler.Add(new Traveler()
                {
                    Id = i,
                    FirstName = "FirstName_" + i,
                    LastName = "LastNane_" + i,
                    Age = i
                });
            }
            #endregion
        }
        #region 
        public List<Traveler> GetAdults()
        {
            var result = Measure(() =>
            {
                var adults = _cache.GetOrCreate<List<Traveler>>("GetAdults", (cacheEntry) =>
                {
                    return _traveler.Where(p => p.Age > 4000).ToList(); ;
                });

                _logger.LogInformation($"Adults.Count: {adults.Count}");

                return adults;
            });

            return result;
        }

        public bool HasAdult()
        {
            var result = Measure(() =>
            {
                var hasAdult = _cache.GetOrCreate<bool>("HasAdult", (cacheEntry) =>
                {
                    return GetAdults().Count > 0;
                });

                _logger.LogInformation($"HasAdult: {hasAdult}");

                return hasAdult;
            });

            return result;
        }
        private T Measure<T>(Func<T> func, [CallerMemberName] string memberName = "")
        {
            var sw = new Stopwatch();
            sw.Start();
            var result = func();
            sw.Stop();
            Console.WriteLine($"{memberName}:\t{sw.Elapsed}");

            return result;
        }
        #endregion

        public bool HasTraveler(string name)
        {
            var result = Measure(() =>
            {
                if (!_validation.IsValidName(name))
                {
                    _logger.LogInformation($"invalid name: {name}");
                    throw new ArgumentException("Invalid name");
                }

                var hasTraveler = _cache.GetOrCreate<bool>("HasTraveler", (cacheEntry) =>
                {
                    return _traveler.Any(p => p.FirstName.ToUpper() == name.ToUpper());
                });

                _logger.LogInformation($"hasTraveler: {hasTraveler}");

                return hasTraveler;
            });

            return result;
        }
      
    }
}
